import sys
import functools
from .colors import *


class Color:

    def __init__(self, color="Default", style="Default"):
        self.color = color
        self.style = style

    def print(self, *objects, sep=' ', end='\n', file=sys.stdout, flush=False):
        print('\033[%s;%sm' % (Style.__dict__[self.style.lower()], Foreground.__dict__[self.color.lower()],), end='',
              flush=True)
        print(*objects, sep=sep, end='', file=file, flush=flush)
        print('\033[0m', end=end, flush=True)

    def __call__(self, func):
        @functools.wraps(func)
        def wrap(*args, **kwargs):
            print('\033[%s;%sm' % (Style.__dict__[self.style.lower()], Foreground.__dict__[self.color.lower()]), end='',
                  flush=True)
            value = func(*args, **kwargs)
            print('\033[0m', end='', flush=True)
            return value

        return wrap

    def setenv(self):
        print('\033[%s;%sm' % (Style.__dict__[self.style.lower()], Foreground.__dict__[self.color.lower()]), end='',
              flush=True)

    def clear(self):
        print('\033[0m', end='', flush=True)

    def __getattr__(self, item):
        if isinstance(item, str):
            return item.title()

    def __repr__(self):
        return "<Color('%s','%s')>" % (self.color.title(), self.style.title())



from .colors import Foreground
from .style import *

def test_style():
    BLACK.print("BLACK")
    RED.print("RED")
    GREEN.print("GREEN")
    YELLOW.print("YELLOW")
    BLUE.print("BLUE")
    MAGENTA.print("MAGENTA")
    CYAN.print("CYAN")
    WHITE.print("WHITE")

    IBLACK.print("IBLACK")
    IRED.print("IRED")
    IGREEN.print("IGREEN")
    IYELLOW.print("IYELLOW")
    IBLUE.print("IBLUE")
    IMAGENTA.print("IMAGENTA")
    ICYAN.print("ICYAN")
    IWHITE.print("IWHITE")

    UBLACK.print("UBLACK")
    URED.print("URED")
    UGREEN.print("UGREEN")
    UYELLOW.print("UYELLOW")
    UBLUE.print("UBLUE")
    UMAGENTA.print("UMAGENTA")
    UCYAN.print("UCYAN")
    UWHITE.print("UWHITE")

    TBLACK.print("TBLACK")
    TRED.print("TRED")
    TGREEN.print("TGREEN")
    TYELLOW.print("TYELLOW")
    TBLUE.print("TBLUE")
    TMAGENTA.print("TMAGENTA")
    TCYAN.print("TCYAN")
    TWHITE.print("TWHITE")

    BBLACK.print("BLACK")
    BRED.print("RED")
    BGREEN.print("GREEN")
    BYELLOW.print("YELLOW")
    BBLUE.print("BLUE")
    BMAGENTA.print("MAGENTA")
    BCYAN.print("CYAN")
    BWHITE.print("WHITE")


def test_color():
    char = "█" * 10
    color_dict = {key: value for key, value in Foreground.__dict__.items() if
                  isinstance(value, int) and key != 'default'}
    for key, value in color_dict.items():
        Color(key).print(char, key.title())




from .xcolor import *
from .style import *
from .test import *

__all__ = [
    "Color",
    "BLACK",
    "RED",
    "GREEN",
    "YELLOW",
    "BLUE",
    "MAGENTA",
    "CYAN",
    "WHITE",
    "BLACKA",
    "REDA",
    "GREENA",
    "YELLOWA",
    "BLUEA",
    "MAGENTAA",
    "CYANA",
    "WHITEA",
    "BLACKB",
    "REDB",
    "GREENB",
    "YELLOWB",
    "BLUEB",
    "MAGENTAB",
    "CYANB",
    "WHITEB",
    "IBLACK",
    "IRED",
    "IGREEN",
    "IYELLOW",
    "IBLUE",
    "IMAGENTA",
    "ICYAN",
    "IWHITE",
    "IBLACKA",
    "IREDA",
    "IGREENA",
    "IYELLOWA",
    "IBLUEA",
    "IMAGENTAA",
    "ICYANA",
    "IWHITEA",
    "IBLACKB",
    "IREDB",
    "IGREENB",
    "IYELLOWB",
    "IBLUEB",
    "IMAGENTAB",
    "ICYANB",
    "IWHITEB",
    "UBLACK",
    "URED",
    "UGREEN",
    "UYELLOW",
    "UBLUE",
    "UMAGENTA",
    "UCYAN",
    "UWHITE",
    "UBLACKA",
    "UREDA",
    "UGREENA",
    "UYELLOWA",
    "UBLUEA",
    "UMAGENTAA",
    "UCYANA",
    "UWHITEA",
    "UBLACKB",
    "UREDB",
    "UGREENB",
    "UYELLOWB",
    "UBLUEB",
    "UMAGENTAB",
    "UCYANB",
    "UWHITEB",
    "TBLACK",
    "TRED",
    "TGREEN",
    "TYELLOW",
    "TBLUE",
    "TMAGENTA",
    "TCYAN",
    "TWHITE",
    "TBLACKA",
    "TREDA",
    "TGREENA",
    "TYELLOWA",
    "TBLUEA",
    "TMAGENTAA",
    "TCYANA",
    "TWHITEA",
    "TBLACKB",
    "TREDB",
    "TGREENB",
    "TYELLOWB",
    "TBLUEB",
    "TMAGENTAB",
    "TCYANB",
    "TWHITEB",
    "BBLACK",
    "BRED",
    "BGREEN",
    "BYELLOW",
    "BBLUE",
    "BMAGENTA",
    "BCYAN",
    "BWHITE",
    "test_color",
    "test_style",
]



from .xcolor import Color

BLACK = Color("Black")
RED = Color("Red")
GREEN = Color("Green")
YELLOW = Color("Yellow")
BLUE = Color("Blue")
MAGENTA = Color("Magenta")
CYAN = Color("Cyan")
WHITE = Color("White")

BLACKA = Color("BlackA")
REDA = Color("RedA")
GREENA = Color("GreenA")
YELLOWA = Color("YellowA")
BLUEA = Color("BlueA")
MAGENTAA = Color("MagentaA")
CYANA = Color("CyanA")
WHITEA = Color("WhiteA")

BLACKB = Color("BlackB")
REDB = Color("RedB")
GREENB = Color("GreenB")
YELLOWB = Color("YellowB")
BLUEB = Color("BlueB")
MAGENTAB = Color("MagentaB")
CYANB = Color("CyanB")
WHITEB = Color("WhiteB")

IBLACK = Color("Black", "Italic")
IRED = Color("Red", "Italic")
IGREEN = Color("Green", "Italic")
IYELLOW = Color("Yellow", "Italic")
IBLUE = Color("Blue", "Italic")
IMAGENTA = Color("Magenta", "Italic")
ICYAN = Color("Cyan", "Italic")
IWHITE = Color("White", "Italic")

IBLACKA = Color("BlackA", "Italic")
IREDA = Color("RedA", "Italic")
IGREENA = Color("GreenA", "Italic")
IYELLOWA = Color("YellowA", "Italic")
IBLUEA = Color("BlueA", "Italic")
IMAGENTAA = Color("MagentaA", "Italic")
ICYANA= Color("CyanA", "Italic")
IWHITEA = Color("WhiteA", "Italic")

IBLACKB = Color("BlackB", "Italic")
IREDB = Color("RedB", "Italic")
IGREENB = Color("GreenB", "Italic")
IYELLOWB = Color("YellowB", "Italic")
IBLUEB = Color("BlueB", "Italic")
IMAGENTAB = Color("MagentaB", "Italic")
ICYANB = Color("CyanB", "Italic")
IWHITEB = Color("WhiteB", "Italic")

UBLACK = Color("Black", "Underline")
URED = Color("Red", "Underline")
UGREEN = Color("Green", "Underline")
UYELLOW = Color("Yellow", "Underline")
UBLUE = Color("Blue", "Underline")
UMAGENTA = Color("Magenta", "Underline")
UCYAN = Color("Cyan", "Underline")
UWHITE = Color("White", "Underline")

UBLACKA = Color("BlackA", "Underline")
UREDA = Color("RedA", "Underline")
UGREENA = Color("GreenA", "Underline")
UYELLOWA = Color("YellowA", "Underline")
UBLUEA = Color("BlueA", "Underline")
UMAGENTAA = Color("MagentaA", "Underline")
UCYANA = Color("CyanA", "Underline")
UWHITEA = Color("WhiteA", "Underline")

UBLACKB = Color("BlackB", "Underline")
UREDB = Color("RedB", "Underline")
UGREENB = Color("GreenB", "Underline")
UYELLOWB = Color("YellowB", "Underline")
UBLUEB = Color("BlueB", "Underline")
UMAGENTAB = Color("MagentaB", "Underline")
UCYANB = Color("CyanB", "Underline")
UWHITEB = Color("WhiteB", "Underline")

TBLACK = Color("Black", "Throughline")
TRED = Color("Red", "Throughline")
TGREEN = Color("Green", "Throughline")
TYELLOW = Color("Yellow", "Throughline")
TBLUE = Color("Blue", "Throughline")
TMAGENTA = Color("Magenta", "Throughline")
TCYAN = Color("Cyan", "Throughline")
TWHITE = Color("White", "Throughline")

TBLACKA = Color("BlackA", "Throughline")
TREDA = Color("RedA", "Throughline")
TGREENA = Color("GreenA", "Throughline")
TYELLOWA = Color("YellowA", "Throughline")
TBLUEA = Color("BlueA", "Throughline")
TMAGENTAA = Color("MagentaA", "Throughline")
TCYANA = Color("CyanA", "Throughline")
TWHITEA = Color("WhiteA", "Throughline")

TBLACKB = Color("BlackB", "Throughline")
TREDB = Color("RedB", "Throughline")
TGREENB = Color("GreenB", "Throughline")
TYELLOWB = Color("YellowB", "Throughline")
TBLUEB = Color("BlueB", "Throughline")
TMAGENTAB = Color("MagentaB", "Throughline")
TCYANB = Color("CyanB", "Throughline")
TWHITEB = Color("WhiteB", "Throughline")

BBLACK = Color("Black", "Bold")
BRED = Color("Red", "Bold")
BGREEN = Color("Green", "Bold")
BYELLOW = Color("Yellow", "Bold")
BBLUE = Color("Blue", "Bold")
BMAGENTA = Color("Magenta", "Bold")
BCYAN = Color("Cyan", "Bold")
BWHITE = Color("White", "Bold")
